

{================ BSDBGRID DISTRIBUTION NOTES ============================================}


{BSDBGRID is a simple, but very effective TDBGRID descendant that can display
each cell in the grid with either a raised, or lowered, 3-D appearance; in addition
to the usual 'flat' appearance. BSDBGRID also provides added functionality in that
the MOUSEUP and MOUSEDOWN events are inherited. }

{COPYRIGHT NOTICE}
{BSDBGRID is distributed as FREEWARE, but remains the COPYRIGHT of
BUSINESS SOFTWARE (UK) (email  ebinfo@compuserve.com ). Business Software grants you the right
to include this compiled component in your DELPHI application, whether COMMERCIAL, SHAREWARE, or
FREEWARE, BUT YOU MAY NOT DISTRIBUTE THIS SOURCE CODE OR ITS COMPILED .DCU  IN ANY FORM OTHER
THAN AS IT EXISTS HERE; COMPLETE WITH THIS NOTICE AND ALL THE TEXT BELOW. BSDBGRID may be included
in any shareware or freeware libraries or compilation disks, provided no charge other than the
usual media cost recovery is made.}

{IF YOU HAVE ANY DOUBTS ABOUT WHETHER YOU MAY LEGALLY USE OR DISTRIBUTE THIS COMPONENT,
CONTACT BUSINESS SOFTWARE BY E-MAIL.}

{VISIT BUSINESS SOFTWARE'S WEB SITE AT HTTP://OURWORLD.COMPUSERVE.COM/HOMEPAGES/EBINFO/  for
more interesting components and applications}

{WARRANTY / ACCEPTANCE OF LIABILITY / INDEMNITY}
{ABSOLUTELY NONE WHATSOEVER}


{INSTALLATION}
{1.Copy this file into your DELPHI/LIB directory, or wherever your library files are kept}
{2.Select OPTIONS|INSTALL COMPONENTS|ADD...}
{3.ADD This file; BSDBGRID.PAS to the INSTALLED UNITS listbox}
{4.Click OK}

{USING BSDBGRID}
{In order to set the effects, a new PROPERTY 'Cellstyle', is published. Cellstyle may
have one of three possible values: csNONE (default), csRAISED, and csLOWERED.
Cellstyle may be set at design time, and will have a global effect on the Grid display,
however, a much more interesting effect can be achieved if the property is altered dynamically
at run time.

For example; to LOWER any cells that contain a numerical value less than zero, and RAISE
any cells with a value greater than 100, add within the 'ondrawdatacell' event handler,
the code to raise or lower the cell.
e.g.

procedure Tform1.BSDBGrid1DrawDataCell(Sender:TObject;const Rect:TRect;
Field:TField;State:TGridDrawState);
const
numbertypes=[ftcurrency,ftfloat,ftinteger,ftsmallint,ftbcd,ftword];

begin
with bsdbgrid1 do
  begin
  cellstyle:=csnone;
  if (field.datatype in numbertypes) then
     begin
     if (field.asfloat<0) then Cellstyle:=csLowered;
     if (field.asfloat>100) then Cellstyle:=csRaised;
     end;

     {if default drawing is off, we still need to do the text drawing;
     you could make the most of it here by using different font colors for
     positives and negatives, etc, But for this demo, here
     is the elementary way:

     Canvas.TextRect(Rect,Rect.left+2,Rect.Top+2,field.asString);


  end;
end;

{=============== END OF DISTRIBUTION NOTES ===================================================}

unit BSDBGrid;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Grids, DBGrids;

type
Tcellstyle=(csRaised,csLowered,csNone);

type
  TBSDBGrid = class(TDBGrid)
  private
    Fcellstyle:Tcellstyle;
    FOnMouseDown:TMouseEvent;
    FOnMouseUp:TMouseEvent;
    procedure setcellstyle(value:Tcellstyle);
  protected
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure DrawCell(ACol, ARow: Longint;ARect: TRect; AState: TGridDrawState); override;
  public
    gridrow:longint;
    gridcol:longint;
    constructor Create(AOwner : TComponent); override;
    destructor destroy; override;
  published
    property Cellstyle:tcellstyle read fcellstyle write setcellstyle;
    property OnMouseDown read FOnMouseDown write FOnMouseDown;
    property OnMouseUp read FOnMouseUp write FOnMouseUp;
  end;

procedure Register;

implementation

constructor TBSDBGrid.Create(AOwner : TComponent);
begin
inherited Create(AOwner);
color:=clBtnFace;
cellstyle:=csNone;
end;

procedure TBSDBGrid.DrawCell(ACol, ARow: Longint; ARect: TRect; AState: TGridDrawState);
var p1,p2:Tcolor;

procedure drawrect;
begin
with ARect do
  begin
  Canvas.Pen.Color :=p1 ;
  Canvas.PolyLine([Point(Left+1, Bottom - 1), Point(Left+1, Top), Point(Right, Top)]);
  Canvas.Pen.Color :=p2;
  Canvas.PolyLine([Point(Left+1, Bottom - 1), Point(right,bottom-1), Point(Right, Top)]);
  end;
end;

procedure cellraised;
begin
p1:=clWindow;
p2:=clBtnShadow;
drawrect;
end;

procedure celllowered;
begin
p2:=clWindow;
p1:=clbtnshadow;
drawrect;
end;

begin
inherited DrawCell(ACol, ARow,ARect,AState);
gridrow:=arow;
gridcol:=acol;
  case fcellstyle of
  csraised:       cellraised;
  cslowered:      celllowered;
  end
end;


procedure TBSDBGrid.Setcellstyle(Value:Tcellstyle);
begin
if (fcellstyle<>value) then fcellstyle:=value;
if (csDesigning in componentstate) then invalidate;
end;

procedure TBSDBGrid.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Assigned(FOnMouseDown) then FOnMouseDown(Self, Button, Shift, X, Y);
  inherited MouseDown(Button, Shift, X, Y);
end;

procedure TBSDBGrid.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Assigned(FOnMouseUp) then FOnMouseUp(Self, Button, Shift, X, Y);
  inherited MouseUp(Button, Shift, X, Y);
end;

destructor TBSDBGrid.destroy;
begin
inherited destroy;
end;

procedure Register;
begin
RegisterComponents('BSoft', [TBSDBGrid]);
end;

end.
